@@ -81,7 +81,7 @@
   (*(node)->pred_func)((pathname), (stat_buf_ptr), (node))
 
 #ifdef STAT_MOUNTPOINTS
-static void init_mounted_dev_list(void);
+static void init_mounted_dev_list(int mandatory);
 #endif
 
 static void process_top_path PARAMS((char *pathname, mode_t mode));
@@ -390,13 +390,14 @@ int
 main (int argc, char **argv)
 {
   int i;
+  const struct parser_table *entry_close, *entry_print, *entry_open;
   const struct parser_table *parse_entry; /* Pointer to the parsing table entry for this expression. */
   struct predicate *cur_pred;
   char *predicate_name;		/* Name of predicate being parsed. */
   int end_of_leading_options = 0; /* First arg after any -H/-L etc. */
-  program_name = argv[0];
-  const struct parser_table *entry_close, *entry_print, *entry_open;
 
+  
+  program_name = argv[0];
 
   /* We call check_nofollow() before setlocale() because the numbers 
    * for which we check (in the results of uname) definitiely have "."
@@ -527,7 +528,7 @@ main (int argc, char **argv)
   assert(entry_close != NULL);
   assert(entry_print != NULL);
   
-  parse_open (entry_open, argv, &argc);
+  parse_openparen (entry_open, argv, &argc);
   parse_begin_user_args(argv, argc, last_pred, predicates);
   pred_sanity_check(last_pred);
   
@@ -583,7 +584,7 @@ main (int argc, char **argv)
   else
     {
       /* `( user-supplied-expression ) -print'. */
-      parse_close (entry_close, argv, &argc);
+      parse_closeparen (entry_close, argv, &argc);
       pred_sanity_check(last_pred);
       parse_print (entry_print, argv, &argc);
       pred_sanity_check(last_pred);
@@ -644,7 +645,7 @@ main (int argc, char **argv)
   if (!options.open_nofollow_available)
     {
 #ifdef STAT_MOUNTPOINTS
-      init_mounted_dev_list();
+      init_mounted_dev_list(0);
 #endif
     }
   
@@ -738,11 +739,15 @@ static size_t num_mounted_devices = 0u;
 
 
 static void
-init_mounted_dev_list()
+init_mounted_dev_list(int mandatory)
 {
   assert(NULL == mounted_devices);
   assert(0 == num_mounted_devices);
   mounted_devices = get_mounted_devices(&num_mounted_devices);
+  if (mandatory && (NULL == mounted_devices))
+    {
+      error(1, 0, "Cannot read list of mounted devices.");
+    }
 }
 
 static void
@@ -754,7 +759,7 @@ refresh_mounted_dev_list(void)
       mounted_devices = 0;
     }
   num_mounted_devices = 0u;
-  init_mounted_dev_list();
+  init_mounted_dev_list(1);
 }
 
 
@@ -1334,7 +1339,7 @@ chdir_back (void)
        * already have it.
        */
       if (NULL == mounted_devices)
-	init_mounted_dev_list();
+	init_mounted_dev_list(1);
 #endif
       
       if (chdir (starting_dir) != 0)
