@@ -74,6 +74,8 @@
 /* for sysconf() */
 #include <unistd.h>
 
+#include <assert.h>
+
 /* COMPAT:  SYSV version defaults size (and has a max value of) to 470.
    We try to make it as large as possible. */
 #if !defined(ARG_MAX) && defined(_SC_ARG_MAX)
@@ -106,19 +108,24 @@ static char *mbstrstr PARAMS ((const char *haystack, const char *needle));
 
 void
 bc_do_insert (const struct buildcmd_control *ctl,
-	      struct buildcmd_state *state,
-	      char *arg, size_t arglen,
-	      const char *prefix, size_t pfxlen,
-	      const char *linebuf, size_t lblen,
-	      int initial_args)
+              struct buildcmd_state *state,
+              char *arg, size_t arglen,
+              const char *prefix, size_t pfxlen,
+              const char *linebuf, size_t lblen,
+              int initial_args)
 {
   /* Temporary copy of each arg with the replace pattern replaced by the
      real arg.  */
   static char *insertbuf;
   char *p;
-  int bytes_left = ctl->arg_max - 1;	/* Bytes left on the command line.  */
+  int bytes_left = ctl->arg_max - 1;    /* Bytes left on the command line.  */
   int need_prefix;
-  
+
+  /* XXX: on systems lacking an upper limit for exec args, ctl->arg_max
+   *      may have been set to LONG_MAX (see bc_get_arg_max()).  Hence
+   *      this xmalloc call may be a bad idea, especially since we are
+   *      adding 1 to it...
+   */
   if (!insertbuf)
     insertbuf = (char *) xmalloc (ctl->arg_max + 1);
   p = insertbuf;
@@ -126,21 +133,21 @@ bc_do_insert (const struct buildcmd_control *ctl,
   need_prefix = 0;
   do
     {
-      size_t len;		/* Length in ARG before `replace_pat'.  */
+      size_t len;               /* Length in ARG before `replace_pat'.  */
       char *s = mbstrstr (arg, ctl->replace_pat);
       if (s)
-	{
-	  need_prefix = 1;
-	  len = s - arg;
-	}
+        {
+          need_prefix = 1;
+          len = s - arg;
+        }
       else
-	{
-	  len = arglen;
-	}
+        {
+          len = arglen;
+        }
       
       bytes_left -= len;
       if (bytes_left <= 0)
-	break;
+        break;
 
       strncpy (p, arg, len);
       p += len;
@@ -148,15 +155,15 @@ bc_do_insert (const struct buildcmd_control *ctl,
       arglen -= len;
 
       if (s)
-	{
-	  bytes_left -= lblen;
-	  if (bytes_left <= 0)
-	    break;
-	  strcpy (p, linebuf);
-	  arg += ctl->rplen;
-	  arglen -= ctl->rplen;
-	  p += lblen;
-	}
+        {
+          bytes_left -= lblen;
+          if (bytes_left <= 0)
+            break;
+          strcpy (p, linebuf);
+          arg += ctl->rplen;
+          arglen -= ctl->rplen;
+          p += lblen;
+        }
     }
   while (*arg);
   if (*arg)
@@ -170,19 +177,44 @@ bc_do_insert (const struct buildcmd_control *ctl,
     }
   
   bc_push_arg (ctl, state,
-	       insertbuf, p - insertbuf,
-	       prefix, pfxlen,
-	       initial_args);
+               insertbuf, p - insertbuf,
+               prefix, pfxlen,
+               initial_args);
 }
 
 static
 void do_exec(const struct buildcmd_control *ctl,
-	     struct buildcmd_state *state)
+             struct buildcmd_state *state)
 {
   (ctl->exec_callback)(ctl, state);
 }
 
 
+/* Return nonzero if there would not be enough room for an additional
+ * argument.  We check the total number of arguments only, not the space
+ * occupied by those arguments.
+ *
+ * If we return zero, there still may not be enough room for the next
+ * argument, depending on its length.
+ */
+static int 
+bc_argc_limit_reached(int initial_args, 
+		      const struct buildcmd_control *ctl,
+		      struct buildcmd_state *state)
+{
+  /* Check to see if we about to exceed a limit set by xargs' -n option */
+  if (!initial_args && ctl->args_per_exec &&
+      ( (state->cmd_argc - ctl->initial_argc) == ctl->args_per_exec))
+    return 1;
+
+  /* We deliberately use an equality test here rather than >= in order
+   * to force a software failure if the code is modified in such a way
+   * that it fails to call this function for every new argument.
+   */
+  return state->cmd_argc == ctl->max_arg_count;
+}
+
+
 /* Add ARG to the end of the list of arguments `cmd_argv' to pass
    to the command.
    LEN is the length of ARG, including the terminating null.
@@ -190,10 +222,10 @@ void do_exec(const struct buildcmd_control *ctl,
 
 void
 bc_push_arg (const struct buildcmd_control *ctl,
-	     struct buildcmd_state *state,
-	     const char *arg, size_t len,
-	     const char *prefix, size_t pfxlen,
-	     int initial_args)
+             struct buildcmd_state *state,
+             const char *arg, size_t len,
+             const char *prefix, size_t pfxlen,
+             int initial_args)
 {
   if (!initial_args)
     state->todo = 1;
@@ -201,34 +233,34 @@ bc_push_arg (const struct buildcmd_control *ctl,
   if (arg)
     {
       if (state->cmd_argv_chars + len > ctl->arg_max)
-	{
-	  if (initial_args || state->cmd_argc == ctl->initial_argc)
-	    error (1, 0, _("can not fit single argument within argument list size limit"));
-	  /* option -i (replace_pat) implies -x (exit_if_size_exceeded) */
-	  if (ctl->replace_pat
-	      || (ctl->exit_if_size_exceeded &&
-		  (ctl->lines_per_exec || ctl->args_per_exec)))
-	    error (1, 0, _("argument list too long"));
-	  do_exec (ctl, state);
-	}
-      if (!initial_args && ctl->args_per_exec &&
-	  state->cmd_argc - ctl->initial_argc == ctl->args_per_exec)
+        {
+          if (initial_args || state->cmd_argc == ctl->initial_argc)
+            error (1, 0, _("can not fit single argument within argument list size limit"));
+          /* option -i (replace_pat) implies -x (exit_if_size_exceeded) */
+          if (ctl->replace_pat
+              || (ctl->exit_if_size_exceeded &&
+                  (ctl->lines_per_exec || ctl->args_per_exec)))
+            error (1, 0, _("argument list too long"));
+          do_exec (ctl, state);
+        }
+      
+      if (bc_argc_limit_reached(initial_args, ctl, state))
 	do_exec (ctl, state);
     }
 
   if (state->cmd_argc >= state->cmd_argv_alloc)
     {
       if (!state->cmd_argv)
-	{
-	  state->cmd_argv_alloc = 64;
-	  state->cmd_argv = (char **) xmalloc (sizeof (char *) * state->cmd_argv_alloc);
-	}
+        {
+          state->cmd_argv_alloc = 64;
+          state->cmd_argv = (char **) xmalloc (sizeof (char *) * state->cmd_argv_alloc);
+        }
       else
-	{
-	  state->cmd_argv_alloc *= 2;
-	  state->cmd_argv = (char **) xrealloc (state->cmd_argv,
-					 sizeof (char *) * state->cmd_argv_alloc);
-	}
+        {
+          state->cmd_argv_alloc *= 2;
+          state->cmd_argv = (char **) xrealloc (state->cmd_argv,
+                                         sizeof (char *) * state->cmd_argv_alloc);
+        }
     }
 
   if (!arg)
@@ -237,10 +269,10 @@ bc_push_arg (const struct buildcmd_control *ctl,
     {
       state->cmd_argv[state->cmd_argc++] = state->argbuf + state->cmd_argv_chars;
       if (prefix)
-	{
-	  strcpy (state->argbuf + state->cmd_argv_chars, prefix);
-	  state->cmd_argv_chars += pfxlen;
-	}
+        {
+          strcpy (state->argbuf + state->cmd_argv_chars, prefix);
+          state->cmd_argv_chars += pfxlen;
+        }
       
       strcpy (state->argbuf + state->cmd_argv_chars, arg);
       state->cmd_argv_chars += len;
@@ -250,10 +282,7 @@ bc_push_arg (const struct buildcmd_control *ctl,
        * conditional on arg!=NULL, since do_exec() 
        * actually calls bc_push_arg(ctl, state, NULL, 0, false).
        */
-      if ((!initial_args
-          && ctl->args_per_exec
-          && (state->cmd_argc - ctl->initial_argc) == ctl->args_per_exec)
-         || state->cmd_argc == ARG_MAX / sizeof (void *) - 1)
+      if (bc_argc_limit_reached(initial_args, ctl, state))
 	do_exec (ctl, state);
     }
 
@@ -281,15 +310,15 @@ mbstrstr (const char *haystack, const char *needle)
 
       memset (&mbstate, 0, sizeof (mbstate_t));
       while (hlen >= nlen)
-	{
-	  if (memcmp (haystack, needle, nlen) == 0)
-	    return (char *) haystack;
-	  step = mbrlen (haystack, hlen, &mbstate);
-	  if (step <= 0)
-	    break;
-	  haystack += step;
-	  hlen -= step;
-	}
+        {
+          if (memcmp (haystack, needle, nlen) == 0)
+            return (char *) haystack;
+          step = mbrlen (haystack, hlen, &mbstate);
+          if (step <= 0)
+            break;
+          haystack += step;
+          hlen -= step;
+        }
       return NULL;
     }
 #endif
@@ -331,7 +360,7 @@ bc_get_arg_max(void)
 
 
 static int cb_exec_noop(const struct buildcmd_control *ctl,
-			 struct buildcmd_state *state)
+                         struct buildcmd_state *state)
 {
   /* does nothing. */
   (void) ctl;
@@ -343,8 +372,14 @@ static int cb_exec_noop(const struct buildcmd_control *ctl,
 void
 bc_init_controlinfo(struct buildcmd_control *ctl)
 {
+  long arg_max = bc_get_arg_max();
+  assert(arg_max > 0);
+  
   ctl->exit_if_size_exceeded = 0;
-  ctl->arg_max = bc_get_arg_max() - 2048; /* a la xargs */
+  ctl->arg_max = arg_max - 2048; /* a la xargs */
+  /* need to subtract 2 on the following line - for Linux/PPC */
+  ctl->max_arg_count = (arg_max / sizeof(void*)) - 2;
+  assert(ctl->max_arg_count > 0);
   ctl->rplen = 0u;
   ctl->replace_pat = NULL;
   ctl->initial_argc = 0;
@@ -355,14 +390,20 @@ bc_init_controlinfo(struct buildcmd_control *ctl)
 
 void
 bc_init_state(const struct buildcmd_control *ctl,
-	      struct buildcmd_state *state,
-	      void *context)
+              struct buildcmd_state *state,
+              void *context)
 {
   state->cmd_argc = 0;
   state->cmd_argv_chars = 0;
   state->cmd_argv = NULL;
   state->cmd_argv_alloc = 0;
+  
+  /* XXX: the following memory allocation is inadvisable on systems
+   * with no ARG_MAX, because ctl->arg_max may actually be LONG_MAX.
+   * Also, adding one to that is a bad idea.
+   */
   state->argbuf = (char *) xmalloc (ctl->arg_max + 1);
+  
   state->cmd_argv_chars = state->cmd_initial_argv_chars = 0;
   state->todo = 0;
   state->usercontext = context;
@@ -370,7 +411,7 @@ bc_init_state(const struct buildcmd_control *ctl,
 
 void 
 bc_clear_args(const struct buildcmd_control *ctl,
-	      struct buildcmd_state *state)
+              struct buildcmd_state *state)
 {
   state->cmd_argc = ctl->initial_argc;
   state->cmd_argv_chars = state->cmd_initial_argv_chars;
